import typing
import unicodedata

from . import _search


__all__ = ['SearchError', 'Ngrams']


SearchError = _search.SearchError


END_MARKER = '$'


def string_trigrams(string: str) -> (int, int, int):
    if len(string) == 0:
        yield END_MARKER, END_MARKER, END_MARKER
    else:
        end = ord(END_MARKER)
        c1 = c2 = end
        for c3 in string:
            c3 = ord(c3)
            yield c1, c2, c3
            c1 = c2
            c2 = c3
        yield c1, c2, end
        yield c2, end, end


class Ngrams(object):
    def __init__(self, path):
        self._file = open(path, 'rb')

    def search(
        self,
        string: str,
        threshold: float = 0.3,
    ) -> typing.List[typing.Tuple[int, float]]:
        # Normalize string
        string = string.lower()
        string = unicodedata.normalize('NFC', string)

        # Build list of trigrams
        trigrams = {}
        for trigram in string_trigrams(string):
            if trigram in trigrams:
                trigrams[trigram] += 1
            else:
                trigrams[trigram] = 1
        trigrams = list(trigrams.items())

        # Call search code
        return _search.search(self._file, trigrams, threshold)



from setuptools import Extension, setup
import os

_DONT_WANT = ('', '0', 'n', 'no', 'false')

def _cython_exts(extensions):
    if os.environ.get('BUILD_CYTHON', '').lower() not in _DONT_WANT:
        from Cython.Build import cythonize
        # Use annotate=True to debug
        return cythonize(extensions)
    else:
        for extension in extensions:
            sources = []
            for sfile in extension.sources:
                path, ext = os.path.splitext(sfile)
                if ext in ('.pyx', '.py'):
                    sfile = path + '.c'
                sources.append(sfile)
            extension.sources[:] = sources
        return extensions


setup(
    name='ngram-search',
    version='0.1',
    packages=['ngram_search'],
    package_data={'ngram_search': ['*.pxd']},
    ext_modules=_cython_exts([Extension('ngram_search._search', ['ngram_search/_search.pyx'])]),
    zip_safe=False,
)
